@@ -77,6 +77,24 @@ static struct ctrl_handle *g_ctrlh;
 static RadioDevice *usrp;
 static RadioInterface *radio;
 
+/* adjusts read timestamp offset to make the viterbi equalizer happy by including the start tail bits */
+template <typename B>
+class rif_va_wrapper : public B {
+	bool use_va;
+
+    public:
+	template <typename... Args>
+	rif_va_wrapper(bool use_va, Args &&...args) : B(std::forward<Args>(args)...), use_va(use_va)
+	{
+	}
+	bool start() override
+	{
+		auto rv = B::start();
+		B::readTimestamp -= use_va ? 20 : 0;
+		return rv;
+	};
+};
+
 /* Create radio interface
  *     The interface consists of sample rate changes, frequency shifts,
  *     channel multiplexing, and other conversions. The transceiver core
@@ -91,17 +109,17 @@ RadioInterface *makeRadioInterface(struct trx_ctx *trx,
 
 	switch (type) {
 	case RadioDevice::NORMAL:
-		radio = new RadioInterface(usrp, trx->cfg.tx_sps,
-					   trx->cfg.rx_sps, trx->cfg.num_chans);
+		radio = new rif_va_wrapper<RadioInterface>(trx->cfg.use_va, usrp, trx->cfg.tx_sps, trx->cfg.rx_sps,
+							   trx->cfg.num_chans);
 		break;
 	case RadioDevice::RESAMP_64M:
 	case RadioDevice::RESAMP_100M:
-		radio = new RadioInterfaceResamp(usrp, trx->cfg.tx_sps,
-						 trx->cfg.rx_sps);
+		radio = new rif_va_wrapper<RadioInterfaceResamp>(trx->cfg.use_va, usrp, trx->cfg.tx_sps,
+								 trx->cfg.rx_sps);
 		break;
 	case RadioDevice::MULTI_ARFCN:
-		radio = new RadioInterfaceMulti(usrp, trx->cfg.tx_sps,
-						trx->cfg.rx_sps, trx->cfg.num_chans);
+		radio = new rif_va_wrapper<RadioInterfaceMulti>(trx->cfg.use_va, usrp, trx->cfg.tx_sps, trx->cfg.rx_sps,
+								trx->cfg.num_chans);
 		break;
 	default:
 		LOG(ALERT) << "Unsupported radio interface configuration";
@@ -465,6 +483,12 @@ int trx_validate_config(struct trx_ctx *trx)
 		return -1;
 	}
 
+	if (trx->cfg.use_va &&
+	    (trx->cfg.egprs || trx->cfg.multi_arfcn || trx->cfg.tx_sps != 4 || trx->cfg.rx_sps != 4)) {
+		LOG(ERROR) << "Viterbi equalizer only works for gmsk with 4 tx/rx samples per symbol!";
+		return -1;
+	}
+
 	return 0;
 }
 
@@ -571,24 +595,14 @@ static void trx_stop()
 static int trx_start(struct trx_ctx *trx)
 {
 	int type, chans;
-	unsigned int i;
-	std::vector<std::string> rx_paths, tx_paths;
 	RadioDevice::InterfaceType iface = RadioDevice::NORMAL;
 
 	/* Create the low level device object */
 	if (trx->cfg.multi_arfcn)
 		iface = RadioDevice::MULTI_ARFCN;
 
-	/* Generate vector of rx/tx_path: */
-	for (i = 0; i < trx->cfg.num_chans; i++) {
-		rx_paths.push_back(charp2str(trx->cfg.chans[i].rx_path));
-		tx_paths.push_back(charp2str(trx->cfg.chans[i].tx_path));
-	}
-
-	usrp = RadioDevice::make(trx->cfg.tx_sps, trx->cfg.rx_sps, iface,
-				 trx->cfg.num_chans, trx->cfg.offset,
-				 tx_paths, rx_paths);
-	type = usrp->open(charp2str(trx->cfg.dev_args), trx->cfg.clock_ref, trx->cfg.swap_channels);
+	usrp = RadioDevice::make(iface, &trx->cfg);
+	type = usrp->open();
 	if (type < 0) {
 		LOG(ALERT) << "Failed to create radio device" << std::endl;
 		goto shutdown;
@@ -651,7 +665,7 @@ int main(int argc, char *argv[])
 		exit(2);
 	}
 
-	rc = telnet_init_dynif(tall_trx_ctx, NULL, vty_get_bind_addr(), OSMO_VTY_PORT_TRX);
+	rc = telnet_init_default(tall_trx_ctx, NULL, OSMO_VTY_PORT_TRX);
 	if (rc < 0)
 		exit(1);
 
